@@ -145,8 +145,7 @@ storage = os.path.join(datahome, "storage")
 
 resfiles = [
 	# Only essentials
-	"argyll_instruments.json",
-	"lang/en.json",
+	"lang/en.yaml",
 	"beep.wav",
 	"camera_shutter.wav",
 	"linear.cal",
@@ -262,6 +261,8 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 			w = int(round(w * scale))
 			h = int(round(h * scale))
 		if parts[-1] == "empty":
+			if wx.VERSION < (3, ):
+				use_mask = True
 			if use_mask and sys.platform == "win32":
 				bmp = wx.EmptyBitmap(w, h)
 				bmp.SetMaskColour(wx.Colour(0, 0, 0))
@@ -365,16 +366,21 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 						img.Rescale(w, h, quality=quality)
 				factors = None
 				if (not inverted and len(parts) > 2 and parts[-3] == "icons" and
+					(ow, oh) != (10, 10) and oname not in ("black_luminance",
+														   "check_all",
+														   "contrast",
+														   "luminance") and
 					max(wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOW)[:3]) < 102):
+					# Automatically invert B&W image if background is dark
+					# (exceptions do apply)
 					if not img:
 						img = bmp.ConvertToImage()
 					if img.IsBW():
 						inverted = True
 				# Invert after resize (avoids jaggies)
-				if inverted:
+				if inverted or color:
 					if not img:
 						img = bmp.ConvertToImage()
-					img.Invert()
 					alpha = wx.SystemSettings.GetColour(wx.SYS_COLOUR_BTNTEXT).alpha
 					if oname in ["applications-system", "color",
 								 "document-open", "document-save-as",
@@ -388,11 +394,18 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 						# Scale 85 to 255 and adjust alpha
 						factors = (3, 3, 3, alpha / 255.0)
 					else:
-						# Only adjust alpha
-						factors = (1, 1, 1, alpha / 255.0)
-				if color or factors:
-					if not img:
-						img = bmp.ConvertToImage()
+						if inverted:
+							img.Invert()
+						if alpha != 255:
+							# Only adjust alpha
+							factors = (1, 1, 1, alpha / 255.0)
+					if factors:
+						R, G, B = factors[:3]
+						if len(factors) > 3:
+							alpha = factors[3]
+						else:
+							alpha = 1.0
+						img = img.AdjustChannels(R, G, B, alpha)
 					if color:
 						# Hex format, RRGGBB or RRGGBBAA
 						R = int(color[0:2], 16) / 255.0
@@ -402,13 +415,7 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 							alpha = int(color[6:8], 16) / 255.0
 						else:
 							alpha = 1.0
-					else:
-						R, G, B = factors[:3]
-						if len(factors) > 3:
-							alpha = factors[3]
-						else:
-							alpha = 1.0
-					img = img.AdjustChannels(R, G, B, alpha)
+						img = img.AdjustChannels(R, G, B, alpha)
 				if img:
 					bmp = img.ConvertToBitmap()
 					if not bmp.IsOk():
@@ -704,20 +711,27 @@ cfg = ConfigParser.RawConfigParser()
 cfg.optionxform = str
 
 valid_ranges = {
-	"3dlut.hdr_peak_luminance": [120.0, 10000.0],
+	"3dlut.hdr_peak_luminance": [100.0, 10000.0],
 	"3dlut.hdr_minmll": [0.0, 0.1],
-	"3dlut.hdr_maxmll": [120.0, 10000.0],
+	"3dlut.hdr_maxmll": [100.0, 10000.0],
 	"3dlut.trc_gamma": [0.000001, 10],
+	"3dlut.hdr_sat": [0.0, 1.0],
+	"3dlut.hdr_hue": [0.0, 1.0],
 	"3dlut.trc_output_offset": [0.0, 1.0],
 	"app.port": [1, 65535],
 	"gamma": [0.000001, 10],
 	"trc": [0.000001, 10],
-	"calibration.ambient_viewcond_adjust.lux": [0, sys.maxint],
-	"calibration.black_luminance": [0.000001, 100000],
+	# Argyll dispcal uses 20% of ambient (in lux,
+	# fixed steradiant of 3.1415) as adapting
+	# luminance, but we assume it already *is*
+	# the adapting luminance. To correct for this,
+	# scale so that dispcal gets the correct value.
+	"calibration.ambient_viewcond_adjust.lux": [0.0, sys.maxint / 5.0],
+	"calibration.black_luminance": [0.000001, 10],
 	"calibration.black_output_offset": [0, 1],
 	"calibration.black_point_correction": [0, 1],
 	"calibration.black_point_rate": [0.05, 20],
-	"calibration.luminance": [0.000001, 100000],
+	"calibration.luminance": [20, 100000],
 	"iccgamut.surface_detail": [1.0, 50.0],
 	"measurement_report.trc_gamma": [0.01, 10],
 	"measurement_report.trc_output_offset": [0.0, 1.0],
@@ -725,6 +739,9 @@ valid_ranges = {
 	"measure.min_display_update_delay_ms": [20, 60000],
 	"multiprocessing.max_cpus": [0, 65],
 	"patterngenerator.apl": [0.0, 1.0],
+	"patterngenerator.ffp_insertion.duration": [0.1, 60.0],
+	"patterngenerator.ffp_insertion.interval": [0.0, 3600.0],
+	"patterngenerator.ffp_insertion.level": [0.0, 1.0],
 	"patterngenerator.quantize_bits": [0, 32],
 	"patterngenerator.resolve.port": [1, 65535],
 	"profile_loader.quantize_bits": [8, 16],
@@ -749,8 +766,8 @@ valid_values = {
 	# collink: xvYCC output encoding is not supported
 	"3dlut.encoding.output": filter(lambda v: v not in ("T", "x", "X"),
 									video_encodings),
-	"3dlut.format": ["3dl", "cube", "eeColor", "icc", "madVR", "mga", "png",
-					 "ReShade", "spi3d"],
+	"3dlut.format": ["3dl", "cube", "dcl", "eeColor", "icc", "madVR", "mga",
+					 "png", "ReShade", "spi3d"],
 	"3dlut.hdr_display": [0, 1],
 	"3dlut.image.layout": ["h", "v"],
 	"3dlut.image.order": ["rgb", "bgr"],
@@ -811,7 +828,7 @@ valid_values = {
 	"whitepoint.colortemp.locus": ["t", "T"]
 }
 
-content_rgb_space = colormath.get_rgb_space("DCI P3")
+content_rgb_space = colormath.get_rgb_space("DCI P3 D65")
 crx, cry = content_rgb_space[2:][0][:2]
 cgx, cgy = content_rgb_space[2:][1][:2]
 cbx, cby = content_rgb_space[2:][2][:2]
@@ -845,8 +862,11 @@ defaults = {
 	"3dlut.hdr_display": 0,
 	"3dlut.hdr_minmll": 0.0,
 	"3dlut.hdr_maxmll": 10000.0,
-	"3dlut.hdr_peak_luminance": 400.0,
+	"3dlut.hdr_maxmll_alt_clip": 1,
+	"3dlut.hdr_peak_luminance": 480.0,
 	"3dlut.hdr_ambient_luminance": 5.0,
+	"3dlut.hdr_sat": 0.5,
+	"3dlut.hdr_hue": 0.5,
 	"3dlut.image.layout": "h",
 	"3dlut.image.order": "rgb",
 	"3dlut.input.profile": "",
@@ -872,7 +892,7 @@ defaults = {
 	"drift_compensation.blacklevel": 0,
 	"drift_compensation.whitelevel": 0,
 	"calibration.ambient_viewcond_adjust": 0,
-	"calibration.ambient_viewcond_adjust.lux": 500.0,
+	"calibration.ambient_viewcond_adjust.lux": 32.0,
 	"calibration.autoload": 0,
 	"calibration.black_luminance": 0.000001,
 	"calibration.black_luminance.backup": 0.000001,
@@ -900,7 +920,7 @@ defaults = {
 	"colorimeter_correction.instrument.reference": None,
 	"colorimeter_correction.measurement_mode": "l",
 	"colorimeter_correction.measurement_mode.reference.adaptive": 1,
-	"colorimeter_correction.measurement_mode.reference.highres": 0,
+	"colorimeter_correction.measurement_mode.reference.highres": 1,
 	"colorimeter_correction.measurement_mode.reference.projector": 0,
 	"colorimeter_correction.measurement_mode.reference": "l",
 	"colorimeter_correction.observer": "1931_2",
@@ -913,9 +933,9 @@ defaults = {
 	# Note: worker.Worker.enumerate_displays_and_ports() overwrites copyright
 	"copyright": "No copyright. Created with %s %s and ArgyllCMS" % (appname, 
 																	 version),
-	"dimensions.measureframe": "0.5,0.5,1.5",
-	"dimensions.measureframe.unzoomed": "0.5,0.5,1.5",
-	"dimensions.measureframe.whitepoint.visual_editor": "0.5,0.5,1.5",
+	"dimensions.measureframe": "0.5,0.5,1.0",
+	"dimensions.measureframe.unzoomed": "0.5,0.5,1.0",
+	"dimensions.measureframe.whitepoint.visual_editor": "0.5,0.5,1.0",
 	"display.number": 1,
 	"display_lut.link": 1,
 	"display_lut.number": 1,
@@ -951,7 +971,7 @@ defaults = {
 	"measurement_mode": "l",
 	"measurement_mode.adaptive": 1,
 	"measurement_mode.backup": "l",
-	"measurement_mode.highres": 0,
+	"measurement_mode.highres": 1,
 	"measurement_mode.projector": 0,
 	"measurement_report.apply_black_offset": 0,
 	"measurement_report.apply_trc": 0,
@@ -987,6 +1007,10 @@ defaults = {
 	"observer.backup": "1931_2",
 	"patterngenerator.apl": .22,
 	"patterngenerator.detect_video_levels": 1,
+	"patterngenerator.ffp_insertion": 0,
+	"patterngenerator.ffp_insertion.duration": 5.0,
+	"patterngenerator.ffp_insertion.interval": 5.0,
+	"patterngenerator.ffp_insertion.level": 0.15,
 	"patterngenerator.prisma.argyll": 0,
 	"patterngenerator.prisma.host": "",
 	"patterngenerator.prisma.preset": "Custom-1",
@@ -1375,7 +1399,7 @@ def get_current_profile(include_display_profile=False):
 	if path:
 		import ICCProfile as ICCP
 		try:
-			profile = ICCP.ICCProfile(path)
+			profile = ICCP.ICCProfile(path, use_cache=True)
 		except (IOError, ICCP.ICCProfileInvalidError), exception:
 			return
 		return profile
@@ -1443,7 +1467,7 @@ def get_standard_profiles(paths_only=False):
 						other_icc.append(os.path.join(dirpath, basename))
 		for path in ref_icc + other_icc:
 			try:
-				profile = ICCP.ICCProfile(path, load=False)
+				profile = ICCP.ICCProfile(path, load=False, use_cache=True)
 			except EnvironmentError:
 				pass
 			except Exception, exception:
@@ -1558,7 +1582,7 @@ def is_profile(filename=None, include_display_profile=False):
 		if os.path.exists(filename):
 			import ICCProfile as ICCP
 			try:
-				profile = ICCP.ICCProfile(filename)
+				profile = ICCP.ICCProfile(filename, use_cache=True)
 			except (IOError, ICCP.ICCProfileInvalidError):
 				pass
 			else:
@@ -1632,7 +1656,7 @@ def initcfg(module=None):
 		cfgnames.append(cfgbasename)
 	else:
 		cfgnames.extend("%s-%s" % (appbasename, othermod) for othermod in
-						("synthprofile", "testchart-editor"))
+						("testchart-editor", ))
 	cfgroots = [confighome]
 	if module == "apply-profiles":
 		cfgroots.append(config_sys)
@@ -1657,13 +1681,17 @@ def initcfg(module=None):
 		safe_print("Warning - could not parse configuration files:\n%s" %
 				   "\n".join(cfgfiles))
 		# Fix Python 2.7 ConfigParser option values being lists instead of
-		# strings in case of a ParsingError. http://bugs.python.org/issue2414
+		# strings in case of a ParsingError. http://bugs.python.org/issue24142
 		all_sections = [ConfigParser.DEFAULTSECT]
 		all_sections.extend(cfg.sections())
 		for section in all_sections:
 			for name, val in cfg.items(section):
 				if isinstance(val, list):
 					cfg.set(section, name, "\n".join(val))
+	finally:
+		if not module and not getcfg("calibration.ambient_viewcond_adjust"):
+			# Reset to default
+			setcfg("calibration.ambient_viewcond_adjust.lux", None)
 
 
 dpiset = False
@@ -1674,36 +1702,148 @@ def set_default_app_dpi():
 	global dpiset
 	if not dpiset and not getcfg("app.dpi", False):
 		# HighDPI support
-		dpiset = True
 		from wxaddons import wx
+		dpiset = True
 		if sys.platform in ("darwin", "win32"):
 			# Determine screen DPI
 			dpi = wx.ScreenDC().GetPPI()[0]
 		else:
-			if "gtk3" in wx.PlatformInfo:
-				from util_os import which
-				txt_scale = 1
-				if which("gsettings"):
-					import subprocess as sp
-					p = sp.Popen(["gsettings", "get", "org.gnome.desktop.interface",
-								  "text-scaling-factor"], stdin=sp.PIPE,
-								 stdout=sp.PIPE, stderr=sp.PIPE)
-					stdout, stderr = p.communicate()
-					if stdout:
-						try:
-							txt_scale = float(stdout)
-						except ValueError:
-							pass
-			else:
-				# Linux. Determine font scaling factor
-				font = wx.Font(256, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL,
-							   wx.FONTWEIGHT_NORMAL)
-				txt_scale = font.GetPixelSize()[0] / 256.0 or 1
-			dpi = int(round(get_default_dpi() * txt_scale))
+			# Linux
+			from util_os import which
+			txt_scale = None
+			# XDG_CURRENT_DESKTOP delimiter is colon (':')
+			desktop = os.getenv("XDG_CURRENT_DESKTOP", "").split(":")
+			if "gtk2" in wx.PlatformInfo:
+				txt_scale = get_hidpi_scaling_factor()
+			elif desktop[0] == "KDE":
+				pass
+				# Nothing to do
+			elif which("gsettings"):
+				import subprocess as sp
+				p = sp.Popen(["gsettings", "get", "org.gnome.desktop.interface",
+							  "text-scaling-factor"], stdin=sp.PIPE,
+							 stdout=sp.PIPE, stderr=sp.PIPE)
+				factor, stderr = p.communicate()
+				try:
+					txt_scale = float(factor)
+				except ValueError:
+					pass
+			dpi = get_default_dpi()
+			if txt_scale:
+				dpi = int(round(dpi * txt_scale))
 		defaults["app.dpi"] = dpi
 	dpiset = True
 
 
+def get_hidpi_scaling_factor():
+	if sys.platform in ("darwin", "win32"):
+		return 1.0  # Handled via app DPI
+	else:
+		# Linux
+		from util_os import which
+		factor = None
+		# XDG_CURRENT_DESKTOP delimiter is colon (':')
+		desktop = os.getenv("XDG_CURRENT_DESKTOP", "").split(":")
+		if desktop[0] == "KDE":
+			# Two env-vars exist: QT_SCALE_FACTOR and
+			# QT_SCREEN_SCALE_FACTORS.
+			# According to documentation[1], the latter is
+			# 'mainly useful for debugging' - that's not how it is
+			# used by KDE though. Changing display scaling via KDE
+			# settings GUI only sets QT_SCREEN_SCALE_FACTORS.
+			# We are thus currently ignoring QT_SCALE_FACTOR.
+			# [1] https://doc.qt.io/qt-5/highdpi.html
+			# QT_SCREEN_SCALE_FACTORS delimiter is semicolon (';')
+			# Format: Mapping of XrandR display names to scale factor
+			# e.g. 'VGA-1=1.5;VGA-2=2.0;'
+			# or just list of scale factors
+			# e.g. '1.5;2.0;'
+			screen_scale_factors = os.getenv("QT_SCREEN_SCALE_FACTORS", "").split(";")
+			if screen_scale_factors:
+				from wxaddons import wx
+				match = False
+				app = wx.GetApp()
+				if app:
+					import RealDisplaySizeMM as RDSMM
+					if not RDSMM._displays:
+						RDSMM.enumerate_displays()
+					top = app.TopWindow
+					if top:
+						tmp = False
+					else:
+						# Create temp frame if no topwindow
+						top = wx.Frame(None)
+						# Move to main window location (and thus screen)
+						x, y = (getcfg("position.x", False),
+								getcfg("position.y", False))
+						if not None in (x, y):
+							top.SetSaneGeometry(x, y)
+						tmp = True
+					# Get wx display
+					wx_display = top.GetDisplay()
+					if tmp:
+						# No longer need our temp frame
+						top.Destroy()
+					# Search for matching display based on geometry
+					pos = wx_display.Geometry[:2]
+					size = wx_display.Geometry[2:]
+					for item in screen_scale_factors:
+						if not item:
+							break
+						if "=" in item:
+							name, factor = item.split("=", 1)
+						else:
+							name, factor = None, item
+						for display in RDSMM._displays:
+							if (display.get("pos") != pos or
+								display.get("size") != size):
+								# No match
+								continue
+							if (name and
+								display.get("xrandr_name") != name):
+								# No match
+								continue
+							# Match found
+							match = True
+							break
+						if match:
+							break
+				if not match:
+					# Use first one
+					factor = screen_scale_factors[0].split("=")[-1]
+		elif which("xrdb"):
+			import subprocess as sp
+			p = sp.Popen(["xrdb", "-query"], stdin=sp.PIPE,
+						 stdout=sp.PIPE, stderr=sp.PIPE)
+			# Format: 'Xft.dpi:        192'
+			stdout, stderr = p.communicate()
+			for line in stdout.splitlines():
+				if line.startswith("Xft.dpi:"):
+					split = line.split()
+					dpi = split[-1]
+					try:
+						factor = float(dpi) / get_default_dpi()
+					except ValueError:
+						factor = None
+		elif which("gsettings"):
+			# GNOME
+			import subprocess as sp
+			p = sp.Popen(["gsettings", "get", "org.gnome.desktop.interface",
+						  "scaling-factor"], stdin=sp.PIPE,
+						 stdout=sp.PIPE, stderr=sp.PIPE)
+			# Format: 'unint32 1'
+			stdout, stderr = p.communicate()
+			split = stdout.split()
+			if split:
+				factor = split[-1]
+		if factor:
+			try:
+				factor = float(factor)
+			except ValueError:
+				factor = 1.0
+		return factor
+
+
 def setcfg(name, value):
 	""" Set an option value in the configuration. """
 	if value is None:
@@ -1794,8 +1934,18 @@ _init_testcharts()
 runtype = runtimeconfig(pyfile)
 
 if sys.platform in ("darwin", "win32") and not os.getenv("SSL_CERT_FILE"):
-	# Use our bundled CA file
-	# https://github.com/certifi/python-certifi/blob/master/certifi/cacert.pem
-	cafile = get_data_path("cacert.pem")
+	try:
+		import certifi
+	except ImportError:
+		cafile = None
+	else:
+		cafile = certifi.where()
+		if cafile and not os.path.isfile(cafile):
+			cafile = None
+	if not cafile:
+		# Use our bundled CA file
+		cafile = get_data_path("cacert.pem")
+		if cafile:
+			cafile = cafile.encode(fs_enc, "replace")
 	if cafile:
-		os.environ["SSL_CERT_FILE"] = cafile.encode(fs_enc, "replace")
+		os.environ["SSL_CERT_FILE"] = cafile
